/**
@license
Copyright (c) 2022 trading_peter
This program is available under Apache License Version 2.0
*/

import { LitElement, html, css } from 'lit';
import { init, registerOverlay, registerIndicator } from 'klinecharts';
import annotation from './overlays/annotation';
import marketOrder from './overlays/marketOrder';
import marketStop from './overlays/marketStop';
import simpleLine from './overlays/simpleLine';
import tag from './overlays/tag';
import { createCustomLineIndicator } from './overlays/custom';

const defaultData = [{
  "close": "0",
  "high": "0",
  "low": "0",
  "open": "0",
  "timestamp": new Date().getTime(),
  "volume": "0"
}];

class TpCandleChart extends LitElement {
  static get styles() {
    return [
      css`
        :host {
          position: relative;
        }

        .chart {
          position: absolute;
          inset: 0;
        }
      `
    ];
  }

  render() {
    return html`
      <div class="chart"></div>
    `;
  }

  static get properties() {
    return {
      // candles: { type: Array },
    };
  }

  initChart() {
    this.chart = init(this.shadowRoot.querySelector('.chart'), {
      styles: {
        grid: {
          show: true,
          horizontal: {
            show: true,
            size: 1,
            color: '#393939',
            style: 'dash',
            dashValue: [2, 2]
          },
          vertical: {
            show: true,
            size: 1,
            color: '#393939',
            style: 'dash',
            dashValue: [2, 2]
          }
        },
        candle: {
          tooltip: {
            labels: ['Time: ', 'Open: ', 'Close: ', 'High: ', 'Low: ', 'Volume: '],
          },
        },
        yAxis: {
          position: 'right'
        }
      }
    });

    this.chart.setOffsetRightDistance(200);
    this.chart.setTimezone('UTC');
    registerOverlay(annotation);
    registerOverlay(simpleLine);
    registerOverlay(marketOrder);
    registerOverlay(marketStop);
    registerOverlay(tag);

    // Apply "empty" data to prevent errors on mouse events.
    this.chart.applyNewData(defaultData);
  }

  firstUpdated() {
    this.initChart();
  }

  removeAllOverlays() {
    this.chart.removeOverlay();
  }

  removeAllIndicators() {
    Array.from(this.chart._panes.keys()).forEach(name => {
      this.chart.removeIndicator(name);
    });
  }

  setPricePrecision(decimals) {
    const { volume } = this.chart.getPriceVolumePrecision();
    this.chart.setPriceVolumePrecision(decimals, volume);
  }

  applyNewData(candles) {
    this.removeAllIndicators();
    this.removeAllOverlays();
    this.chart.applyNewData(Array.isArray(candles) && candles.length > 0 ? candles : defaultData);
    this.buildCandleIndex();
    this.chart.resize();
  }

  buildCandleIndex() {
    this.candleIndex = new Map();
    const list = this.chart.getDataList();

    for (const candle of list) {
      this.candleIndex.set(candle.timestamp, candle);
    }
  }

  resize() {
    this.chart.resize();
  }

  setChartStyles(styles) {
    this.chart.setStyles(styles);
  }

  setBarSpace(space) {
    this.chart.setBarSpace(space);
  }

  setRightDistance(distance) {
    this.chart.setOffsetRightDistance(distance);
  }

  hasIndicator(name) {
    return !!this.chart.getIndicatorByPaneId(name);
  }

  createIndicator(name, data, styles = {}, opts) {
    const ind = createCustomLineIndicator(name, data, styles);
    registerIndicator(ind);
    this.chart.createIndicator(name, true, opts || { id: 'candle_pane' }, () => {})
  }

  drawMarketOrder(ts, price, direction) {
    const color = direction === 'buy' ? '#1AE51A' : '#ff4a13'

    this.chart.createOverlay({
      name: 'marketOrder',
      points: [
        { timestamp: ts, value: price }
      ],
      extendData: () => ({ candle: this.candleIndex.get(ts), direction, color, textColor: '#000000' })
    })
  }

  drawMarketStop(ts, price, status, direction) {
    this.chart.createOverlay({
      name: 'marketStop',
      points: [
        { timestamp: ts, value: price }
      ],
      extendData: () => ({ candle: this.candleIndex.get(ts), direction, color: '#ff4a13', status, textColor: '#000000' })
    })
  }

  drawSimpleLine(points, styles, pane = 'candle_pane') {
    this.chart.createOverlay({
      name: 'simpleLine',
      points,
      styles: {
        line: styles
      },
      lock: true,
    }, pane);
  }

  drawAnnotation(ts, price, text, vAlign) {
    this.chart.createOverlay({
      name: 'annotation',
      points: [
        { timestamp: ts, value: price }
      ],
      extendData: () => ({ text, vAlign })
    })
  }

  drawTag(ts, styles) {
    const candle = this.candleIndex.get(ts);

    if (!candle) {
      return
    }

    this.chart.createOverlay({
      name: 'tag',
      points: [
        { timestamp: ts }
      ],
      extendData: () => ({ candle, color: '#ff4a13', textColor: '#000000', ...styles })
    })
  }
}

window.customElements.define('tp-candle-chart', TpCandleChart);
